/**
 * <pre>标签javascript语法高亮显示插件，目的是对写在<pre>标签的内的源代码示例自动加上语法高亮。 Library v1.0.0
 *
 * 此插件功能是通天塔层楼滚动到可见区域时发送埋点。
 *
 * @param {Object} param              入参
 * @param {String} param.el           需要监听的DOM节点
 * @param {String} param.theme        指定主题
 *
 *
 * 文档：https://gitlab.com/fekits/mc-floor
 *
 * 版本：v1.0.0
 *
 * 作者：xiaojunbo
 *
 * 兼容：IE8+
 *
 */
!(function (root, factory) {
  if (typeof define === 'function' && define.amd) {
    define([], factory);
  } else if (typeof module === 'object' && module.exports) {
    module.exports = factory();
  } else {
    root.McTinting = factory();
  }
}(this, function () {
  var McTinting = function() {
    var param = arguments[0] !== (void 0) ? arguments[0] : {
      el: null,
      theme: null
    };
    this.el = param.el || 'pre';
    this.theme = param.theme || 'default';
    this.init();
  };
  McTinting.prototype.init = function() {
    var $__1 = this;
    this.aPreDom = document.querySelectorAll(this.el) || [];
    this.aPreDom = [].slice.apply(this.aPreDom).filter(function(item) {
      return item.getAttribute('tinted') === null;
    });
    var preNum = this.aPreDom.length;
    var $__2 = function(i) {
      (function() {
        var theDom = $__1.aPreDom[i];
        theDom.setAttribute('theme', $__1.theme);
        theDom.setAttribute('tinted', 'true');
        var theLang = theDom.getAttribute('language');
        var newCode = theDom.innerHTML;
        if (theLang === 'html') {
          newCode = newCode.replace(/</g, '&lt;').replace(/>/g, '&gt;');
          var cssCode = [];
          newCode = newCode.replace(/(&lt;style\s*&gt;)([\s\S]*?)(&lt;\/style&gt;)/g, function(g0, g1, g2, g3) {
            cssCode.push($__1.preCSS(g2));
            return (g1 + "{__CSS_TINTING" + cssCode.length + "__}" + g3);
          });
          var jsCode = [];
          newCode = newCode.replace(/(&lt;script\s*&gt;)([\s\S]*?)(&lt;\/script&gt;)/g, function(g0, g1, g2, g3) {
            jsCode.push($__1.preJS(g2));
            return (g1 + "{__JS_TINTING" + jsCode.length + "__}" + g3);
          });
          newCode = $__1.preHTML(newCode);
          jsCode.forEach(function(item, index) {
            var reg = new RegExp(("{__JS_TINTING" + (index + 1) + "__}"), 'g');
            newCode = newCode.replace(reg, item);
          });
          cssCode.forEach(function(item, index) {
            var reg = new RegExp(("{__CSS_TINTING" + (index + 1) + "__}"), 'g');
            newCode = newCode.replace(reg, item);
          });
        }
        if (theLang === 'css') {
          newCode = $__1.preCSS(newCode);
        }
        if (theLang === 'javascript') {
          newCode = $__1.preJS(newCode);
        }
        theDom.innerHTML = ("<code class=\"mc-tinting-area\"><code class=\"mc-tinting-body\">" + newCode + "</code></code>");
      })();
    };
    for (var i = 0; i < preNum; i++) {
      $__2(i);
    }
  };
  McTinting.prototype.refresh = function() {
    this.init();
  };
  McTinting.prototype.preHTML = function() {
    var code = arguments[0] !== (void 0) ? arguments[0] : '';
    var comCode = [];
    code = code.replace(/&lt;!--(.|\n)*?--&gt;/g, function(g1) {
      comCode.push(("<code class=\"h-com\">" + g1 + "</code>"));
      return ("{__COM_TINTING" + comCode.length + "__}");
    });
    var keyCode = [];
    code = code.replace(/(\s\w*)(=)(["'][^"']*["'])/g, function(g0, g1, g2, g3) {
      keyCode.push(("<code class=\"h-key\">" + g1 + "</code><code class=\"h-equ\">" + g2 + "</code><code class=\"h-str\">" + g3 + "</code>"));
      return ("{__KEY_TINTING" + keyCode.length + "__}");
    });
    var tagCode = [];
    code = code.replace(/(&lt;)(\/?[a-zA-Z]+)([\s\S]*?)(\/?&gt;)/g, function(g0, g1, g2, g3, g4) {
      tagCode.push(("<code class=\"h-tag\"><code class=\"h-bra\">" + g1 + "</code>" + g2 + "</code>" + g3 + "<code class=\"h-tag\"><code class=\"h-bra\">" + g4 + "</code></code>"));
      return ("{__TAG_TINTING" + tagCode.length + "__}");
    });
    tagCode.forEach(function(item, index) {
      var reg = new RegExp(("{__TAG_TINTING" + (index + 1) + "__}"), 'g');
      code = code.replace(reg, item);
    });
    keyCode.forEach(function(item, index) {
      var reg = new RegExp(("{__KEY_TINTING" + (index + 1) + "__}"), 'g');
      code = code.replace(reg, item);
    });
    comCode.forEach(function(item, index) {
      var reg = new RegExp(("{__COM_TINTING" + (index + 1) + "__}"), 'g');
      code = code.replace(reg, item);
    });
    return code;
  };
  McTinting.prototype.preCSS = function() {
    var code = arguments[0] !== (void 0) ? arguments[0] : '';
    var comData = [];
    code = code.replace(/(^|[^\b(http:|https:|"|')])(\/\/[^:][^\n]*)/g, function(S0, S1, S2) {
      comData.push(("<code class=\"c-err\">" + S2 + "</code>"));
      return (S1 + "{__COM_TINTING" + comData.length + "__}");
    });
    code = code.replace(/(\/\*[^*][\s\S]*?\*\/)/g, function(S1) {
      comData.push(("<code class=\"c-com\">" + S1 + "</code>"));
      return ("{__COM_TINTING" + comData.length + "__}");
    });
    var objData = [];
    code = code.replace(/({\s?[^{}]*\s})/g, function(S0, S1) {
      S1 = S1.replace(/([\w-]+)(\s?:\s?)([^;\n}]*)([;}]?)/g, function(S0, S1, S2, S3, S4) {
        objData.push(("<code class=\"c-key\">" + S1 + "</code><code class=\"c-sym\">" + S2 + "</code><code class=\"c-val\">" + S3 + "</code><code class=\"c-sym\">" + S4 + "</code>"));
        return ("{__OBJ_TINTING" + objData.length + "__}");
      });
      return S1;
    });
    var tagData = [];
    code = code.replace(/([:\w-*#.@&]+)(\s?,|\s?{|\s?\s|\s?>)/g, function(S0, S1, S2) {
      tagData.push(("<code class=\"c-tag\">" + S1 + "</code>"));
      return ("{__TAG_TINTING" + tagData.length + "__}" + S2);
    });
    tagData.forEach(function(item, index) {
      var reg = new RegExp(("{__TAG_TINTING" + (index + 1) + "__}"), 'g');
      code = code.replace(reg, item);
    });
    objData.forEach(function(item, index) {
      var reg = new RegExp(("{__OBJ_TINTING" + (index + 1) + "__}"), 'g');
      code = code.replace(reg, item);
    });
    comData.forEach(function(item, index) {
      var reg = new RegExp(("{__COM_TINTING" + (index + 1) + "__}"), 'g');
      code = code.replace(reg, item);
    });
    return code;
  };
  McTinting.prototype.preJS = function(code) {
    var $__1 = this;
    function regCode(S3) {
      var S3Data = [];
      S3 = S3.replace(/(\\(w|W|d|D|s|S|b|B|0|n|f|r|t|v|xxx|xdd|uxxxx))/g, function(S1) {
        S3Data.push(("<code class=\"j-met\">" + S1 + "</code>"));
        return ("{__REG_S3DATA" + S3Data.length + "__}");
      });
      S3 = S3.replace(/\\(\\|\^|\.|\/|\||\*|\(|\)|\[|]|\+|\?|\$)/g, function(S1) {
        S3Data.push(("<code class=\"j-esc\">" + S1 + "</code>"));
        return ("{__REG_S3DATA" + S3Data.length + "__}");
      });
      S3 = S3.replace(/(\^|\|)/g, function(s1) {
        S3Data.push(("<code class=\"j-spe\">" + s1 + "</code>"));
        return ("{__REG_S3DATA" + S3Data.length + "__}");
      });
      S3 = S3.replace(/(\(|\)|\[|])/g, function(s1) {
        S3Data.push(("<code class=\"j-bra\">" + s1 + "</code>"));
        return ("{__REG_S3DATA" + S3Data.length + "__}");
      });
      S3 = S3.replace(/(\+|\*|\?)/g, function(s1) {
        S3Data.push(("<code class=\"j-ope\">" + s1 + "</code>"));
        return ("{__REG_S3DATA" + S3Data.length + "__}");
      });
      S3Data.forEach(function(item, index) {
        var reg = new RegExp(("{__REG_S3DATA" + (index + 1) + "__}"), 'g');
        S3 = S3.replace(reg, item);
      });
      return S3;
    }
    (function() {
      var tasks = [];
      code = code.replace(/</g, '&lt;').replace(/>/g, '&gt;');
      var comData = [];
      code = code.replace(/((^|[^(http:|https:|"|')])\/\/[^:][^\n]*)/g, function($1) {
        comData.push(("<code class=\"j-com\">" + $1 + "</code>"));
        return ("{__COM_TINTING" + comData.length + "__}");
      });
      code = code.replace(/(\/\*[^*][\s\S]*?\*\/)/g, function(g1) {
        comData.push(("<code class=\"j-com\">" + g1 + "</code>"));
        return ("{__COM_TINTING" + comData.length + "__}");
      });
      code = code.replace(/(\/\*\*[\s\S]*?\*\/)/g, function($1) {
        $1 = $1.replace(/(@param)(\s+)/g, '<code class="j-com">$1</code>$2');
        comData.push(("<code class=\"j-com\">" + $1 + "</code>"));
        return ("{__COM_TINTING" + comData.length + "__}");
      });
      tasks.unshift(comData);
      var regData = [];
      code = code.replace(/(\.replace\()(\/)([^\n]*)(\/)([gimsuy]?[gimsuy]?[gimsuy]?[gimsuy]?[gimsuy]?[gimsuy]?,)/g, function(S0, S1, S2, S3, S4, S5) {
        S3 = regCode(S3);
        regData.push(("<code class=\"j-reg\">" + S2 + "</code><code class=\"j-str\">" + S3 + "</code><code class=\"j-reg\">" + S4 + "</code><code class=\"j-str\">" + S5 + "</code>"));
        return (S1 + "{__REG_TINTING" + regData.length + "__}");
      });
      code = code.replace(/(\/)([^\n]*)(\/)(\.test\()/g, function(S0, S1, S2, S3, S4) {
        S2 = regCode(S2);
        regData.push(("<code class=\"j-reg\">" + S1 + "<code class=\"j-str\">" + S2 + "</code>" + S3 + "</code>"));
        return ("{__REG_TINTING" + regData.length + "__}" + S4);
      });
      code = code.replace(/(new\sRegExp\()('|"|`)([^\n]*)(\2)(\s?,\s?)('|"|`)(g?|i?|m?|s?|u?|y?)(\6)/g, function(S0, S1, S2, S3, S4, S5, S6, S7, S8) {
        S2 = regCode(S2);
        regData.push(("<code class=\"j-reg\"><code class=\"j-str\">" + S2 + "</code><code class=\"j-str\">" + S3 + "</code><code class=\"j-sym\">" + S4 + "</code><code class=\"j-str\">" + S5 + S6 + S7 + S8 + "</code></code>"));
        return (S1 + "{__REG_TINTING" + regData.length + "__}");
      });
      tasks.unshift(regData);
      var strData = [];
      code = code.replace(/(`)[^\n(\1)]*?\1/g, function(A1) {
        A1 = $__1.preHTML(A1);
        A1 = A1.replace(/(\${)([^{}]*)(})/g, function(B0, B1, B2, B3) {
          B2 = B2.replace(/(\.)(\w*)(\b)/g, function(C0, C1, C2, C3) {
            return (C1 + "<code class=\"j-sub\">" + C2 + "</code>" + C3);
          });
          return ("<code class=\"j-var\"><code class=\"j-tpl-var\">" + B1 + "</code>" + B2 + "<code class=\"j-tpl-var\">" + B3 + "</code></code>");
        });
        strData.push(("<code class=\"j-tpl\">" + A1 + "</code>"));
        return ("{__STR_TINTING" + strData.length + "__}");
      });
      code = code.replace(/('|")[^\n(\1)]*?\1/g, function($1) {
        strData.push(("<code class=\"j-str\">" + $1 + "</code>"));
        return ("{__STR_TINTING" + strData.length + "__}");
      });
      tasks.unshift(strData);
      var funData = [];
      code = code.replace(/(\.)(\w+)(\s?\()/g, function(S0, S1, S2, S3) {
        funData.push(("<code class=\"j-fun\">" + S2 + "</code>"));
        return (S1 + "{__FUN_TINTING" + funData.length + "__}" + S3);
      });
      code = code.replace(/([^\\/]\b)(\w+)(\s?=\s?\()/g, function(S0, S1, S2, S3) {
        funData.push(("<code class=\"j-fun\">" + S2 + "</code>"));
        return (S1 + "{__FUN_TINTING" + funData.length + "__}" + S3);
      });
      code = code.replace(/(=\s?)((?!function)\w+)(\s?\()/g, function(S0, S1, S2, S3) {
        funData.push(("<code class=\"j-fun\">" + S2 + "</code>"));
        return (S1 + "{__FUN_TINTING" + funData.length + "__}" + S3);
      });
      code = code.replace(/([^\\/]\b)(\w+)(\s?:\s?(function)?\s?\()/g, function(S0, S1, S2, S3) {
        funData.push(("<code class=\"j-fun\">" + S2 + "</code>"));
        return (S1 + "{__FUN_TINTING" + funData.length + "__}" + S3);
      });
      code = code.replace(/([^\\/]\b)((?!if|for|switch|while|function)\w+)(\s?\()/g, function(S0, S1, S2, S3) {
        funData.push(("<code class=\"j-fun\">" + S2 + "</code>"));
        return (S1 + "{__FUN_TINTING" + funData.length + "__}" + S3);
      });
      var varData = [];
      code = code.replace(/(\b|\s)(let\s+|var\s+|import\s+)(\w+)(\s*(=|from)\s*)/gm, function(S0, S1, S2, S3, S4) {
        varData.push(("<code class=\"j-var\">" + S3 + "</code>"));
        return ("" + S1 + S2 + "{__VAR_TINTING" + varData.length + "__}" + S4);
      });
      var retData = [];
      code = code.replace(/(\s|\b)(let|var|const|import|from|ture|false|class|new|this|export\sdefault|function|return|for|if|else|else\sif)(\s+|\b)/g, function(S0, S1, S2, S3) {
        retData.push(("<code class=\"j-ret\">" + S2 + "</code>"));
        return (S1 + "{__RET_TINTING" + retData.length + "__}" + S3);
      });
      var keyData = [];
      code = code.replace(/([^=][,{\s\n]+)([^,\n\s.({})':=]*)(\s*:)/g, function(S0, S1, S2, S3) {
        keyData.push(("<code class=\"j-key\">" + S2 + "</code>"));
        return (S1 + "{__KEY_TINTING" + keyData.length + "__}" + S3);
      });
      var subData = [];
      code = code.replace(/(\.)(\w*)(\b)/g, function(S0, S1, S2, S3) {
        subData.push(("<code class=\"j-sub\">" + S2 + "</code>"));
        return (S1 + "{__SUB_TINTING" + subData.length + "__}" + S3);
      });
      var numData = [];
      code = code.replace(/(\s|\+\s?|\+=\s?|-\s?|-=\s?|\*\s?|\*=\s?|\/\s?|\/=\s?|=\s?|=\+\s?|=-\s?|=\*\s?|=\/\s?|:\s?|,\s?)(-?\d+\.?\d*)(\b)/g, function(S0, S1, S2, S3) {
        numData.push(("<code class=\"j-num\">" + S2 + "</code>"));
        return (S1 + "{__NUM_TINTING" + numData.length + "__}" + S3);
      });
      var objData = [];
      code = code.replace(/(\b[a-zA-Z0-9_]*)(\.)/g, function(S0, S1, S2) {
        objData.push(("<code class=\"j-obj\">" + S1 + "</code>"));
        return ("{__OBJ_TINTING" + objData.length + "__}" + S2);
      });
      var cooData = [];
      code = code.replace(/(===|!==|==|&lt;=|&gt;=|=&gt;|!=|&lt;|&gt;|\+\+|\|\||=|\?|:|;|,)/g, function(S1) {
        cooData.push(("<code class=\"j-coo\">" + S1 + "</code>"));
        return ("{__COO_TINTING" + cooData.length + "__}");
      });
      cooData.forEach(function(item, index) {
        var reg = new RegExp(("{__COO_TINTING" + (index + 1) + "__}"), 'g');
        code = code.replace(reg, item);
      });
      objData.forEach(function(item, index) {
        var reg = new RegExp(("{__OBJ_TINTING" + (index + 1) + "__}"), 'g');
        code = code.replace(reg, item);
      });
      numData.forEach(function(item, index) {
        var reg = new RegExp(("{__NUM_TINTING" + (index + 1) + "__}"), 'g');
        code = code.replace(reg, item);
      });
      subData.forEach(function(item, index) {
        var reg = new RegExp(("{__SUB_TINTING" + (index + 1) + "__}"), 'g');
        code = code.replace(reg, item);
      });
      keyData.forEach(function(item, index) {
        var reg = new RegExp(("{__KEY_TINTING" + (index + 1) + "__}"), 'g');
        code = code.replace(reg, item);
      });
      retData.forEach(function(item, index) {
        var reg = new RegExp(("{__RET_TINTING" + (index + 1) + "__}"), 'g');
        code = code.replace(reg, item);
      });
      varData.forEach(function(item, index) {
        var reg = new RegExp(("{__VAR_TINTING" + (index + 1) + "__}"), 'g');
        code = code.replace(reg, item);
      });
      funData.forEach(function(item, index) {
        var reg = new RegExp(("{__FUN_TINTING" + (index + 1) + "__}"), 'g');
        code = code.replace(reg, item);
      });
      strData.forEach(function(item, index) {
        var reg = new RegExp(("{__STR_TINTING" + (index + 1) + "__}"), 'g');
        code = code.replace(reg, item);
      });
      regData.forEach(function(item, index) {
        var reg = new RegExp(("{__REG_TINTING" + (index + 1) + "__}"), 'g');
        code = code.replace(reg, item);
      });
      comData.forEach(function(item, index) {
        var reg = new RegExp(("{__COM_TINTING" + (index + 1) + "__}"), 'g');
        code = code.replace(reg, item);
      });
    })();
    return code;
  };
  return McTinting;
}));
